/*
Automate de commande de passage à niveau PaN
CMFA 1er Mars 2015
Dominique Bultez
Version 0.9

Le materiel pour 2 voies de circulation dont 1 droite et 1 divisée par aiguille coté gauche du PaN
- 2 ILS (reed) par voie avant passage à niveau, 2 ILS (reed) par voie après passage à niveau
- soit 6 au total
- un passage à niveau PAN à commande digitale ON/OFF moteur Fulgurex alimente quelques secondes
- circulation possible dans les 2 sens sur les 2 voies, plusieurs trains en meme temps
- la barriere ne s'ouvre que quand toutes les voies sont liberees
- 2 feux rouge a Led commandees en PWM pour plus de realisme
- seuls les ILS marques "0xxx" sont autorises  ouvrir la barriere si tout est libre ailleurs
- les ILS marques "Fxxx" ferment la barriere quand un convoi va vers le passage a niveau

  O1-29G --- F1-29G ---\     /--- F1-29D    --- O1-29D    (voie interieure 1) 
                         PaN
  O2-30G --- F2-30G ---/     \--- F2-33031D --- O2-3031D  (voie exterieure 2)
  O2-31G --- F2-31G -/
  
*/
#define VERSION "V0.8 01.03-2014"
#define DEBUG      // a mettre en commentaire pour enlever les echanges avec la console

// Pins utilisées pour les connexions aux detecteurs Reed
#define ReedA1    2  //RA1 = O1-29G
#define ReedB1    3  //RB1 = F1-29G
#define ReedC1    4  //RB2 = F1-29D
#define ReedD1    5  //RD2 = O1-29D
#define ReedA2    6  //RA2 = O2-30G
#define ReedB2    7  //RB2 = F2-30G
#define ReedC2    8  //RC2 = F2-3031D
#define ReedD2    9  //RD2 = O2-3031D
#define ReedA3   A2  //RA3 = O2-31G
#define ReedB3   A3  //RB3 = F2-31G

// Pins utilisées pour la Led et le PAN
#define LED      13    // on board
#define Feux     11    // PWM ou digital OUT
#define PANPWM   12    // not used
#define PAN1     A0    //digital OUT IN2 relai
#define PAN2     A1    //digital OUT IN1 relai
#define PANMOVE  5000  // duree d'activation du moteur Fulgurex

#include <Bounce2.h>  // librairie de gestion des contacts Reed

// Instantiate 8 Bounce objects with a 10 millisecond debounce time
Bounce bounceA1 = Bounce(); 
Bounce bounceB1 = Bounce(); 
Bounce bounceC1 = Bounce(); 
Bounce bounceD1 = Bounce(); 
Bounce bounceA2 = Bounce(); 
Bounce bounceB2 = Bounce(); 
Bounce bounceC2 = Bounce(); 
Bounce bounceD2 = Bounce(); 
Bounce bounceA3 = Bounce();
Bounce bounceB3 = Bounce();

// Variables
boolean RA1, RA2, RB1, RB2, RC1, RC2, RD1, RD2, RA3, RB3 = false;
int etat1, etat2 = 0;  // libre
boolean dir1, dir2 = false;
boolean etat_PAN = true;  // true=ouvert, false=fermé
char c;
unsigned long TimerFeux, TimerPAN;
int fadeValue;
int fading = 10;  // 10 croit/decroit ou 1000 allume/eteint
int cycle = 0;   // 0 eteint, 1 croit, 2 allumé, 3 decroit
int panValue = PANMOVE; //durée impulsion ouverture et fermeture pour Fulgurex
boolean Fulgurex = false;  // true seulement pour envoyer l'impulsion


// SETUP
void setup() {
  // entrées
  pinMode(ReedA1,INPUT_PULLUP);
  pinMode(ReedB1,INPUT_PULLUP);
  pinMode(ReedC1,INPUT_PULLUP);
  pinMode(ReedD1,INPUT_PULLUP);
  pinMode(ReedA2,INPUT_PULLUP);
  pinMode(ReedB2,INPUT_PULLUP);
  pinMode(ReedC2,INPUT_PULLUP);
  pinMode(ReedD2,INPUT_PULLUP);
  pinMode(ReedA3,INPUT_PULLUP);
  pinMode(ReedB3,INPUT_PULLUP);
  // sorties
  pinMode(LED,OUTPUT);
  pinMode(Feux,OUTPUT);
  pinMode(PAN1,OUTPUT);
  pinMode(PAN2,OUTPUT);
  digitalWrite(PAN1, HIGH);
  digitalWrite(PAN2, HIGH);

  bounceA1.attach(ReedA1);  //définition des objets bounce
  bounceB1.attach(ReedB1);
  bounceC1.attach(ReedC1);
  bounceD1.attach(ReedD1);
  bounceA2.attach(ReedA2);
  bounceB2.attach(ReedB2);
  bounceC2.attach(ReedC2);
  bounceD2.attach(ReedD2);
  bounceA3.attach(ReedA3);
  bounceB3.attach(ReedB3);
  bounceA1.interval(50);
  bounceB1.interval(50);
  bounceC1.interval(50);
  bounceD1.interval(50);
  bounceA2.interval(50);
  bounceB2.interval(50);
  bounceC2.interval(50);
  bounceD2.interval(50);
  bounceA3.interval(50);
  bounceB3.interval(50);
  
  TimerFeux = millis();
  
#if defined(DEBUG)
  Serial.begin(9600);  // console de debugging
  Serial.println(VERSION);
#endif

  delay(2000);
}

// LOOP
void loop() {
 // Update the debouncer
  bounceA1.update ( );
  bounceB1.update ( );
  bounceC1.update ( );
  bounceD1.update ( );
  bounceA2.update ( );
  bounceB2.update ( );
  bounceC2.update ( );
  bounceD2.update ( );
  bounceA3.update ( );
  bounceB3.update ( );
 
   // Call code if Bounce fell (transition from HIGH to LOW) :
   if ( bounceA1.fell() ) {
#if defined(DEBUG)
Serial.print("O1-29G");
#endif
     action1();    
   }
  
   if ( bounceB1.fell() ) {
#if defined(DEBUG)
Serial.print("F1-29G");
#endif
     action2();    
   }
  
   if ( bounceC1.fell() ) {
#if defined(DEBUG)
Serial.print("F1-29D");
#endif
     action3();    
   }
  
   if ( bounceD1.fell() ) {
#if defined(DEBUG)
Serial.print("O1-29D");
#endif
     action4();    
   }
  
   if ( bounceA2.fell() ) {
#if defined(DEBUG)
Serial.print("O2-30G");
#endif
     action5();    
   }
  
   if ( bounceA3.fell() ) {
#if defined(DEBUG)
Serial.print("O2-31G");
#endif
     action5();    
   }
  
   if ( bounceB2.fell() ) {
#if defined(DEBUG)
Serial.print("F2-30G");
#endif
     action6();    
   }
  
   if ( bounceB3.fell() ) {
#if defined(DEBUG)
Serial.print("F2-31G");
#endif
     action6();    
   }
  
  if ( bounceC2.fell() ) {
#if defined(DEBUG)
Serial.print("F2-3031D");
#endif
     action7();    
   }
  
   if ( bounceD2.fell() ) {
#if defined(DEBUG)
Serial.print("O2-3031D");
#endif
     action8();    
   }
  
 
 // Turn on or off the LED
   digitalWrite(LED, etat_PAN );
   
   if ((TimerFeux + fading) < millis())
   {
     TimerFeux = TimerFeux + fading;
     if (!etat_PAN)
     {
       FeuxON();
     }
   }
 
 // commande de barrière (impulsion de duree panValue)  
   if (Fulgurex)
   {
     if ((TimerPAN + panValue) < millis())  // depassement timer
     {
       Fulgurex = false;
       digitalWrite(PAN2, HIGH);       // arret relais Fulgurex
       digitalWrite(PAN1, HIGH);
       if (etat_PAN) 
       {
         FeuxOFF();                  // extinction feux  la fin de l'ouverture
       }
       #if defined(DEBUG)
       Serial.print(" Passage a Niveau ");
       if (etat_PAN) Serial.println("Ouvert "); else Serial.println("Ferme ");
       #endif
     }
   }
 
#if defined(DEBUG)
if (Serial.available()) {//DEBUG TEST équivalent clavier des capteurs Reed
   c = Serial.read();
   Serial.print(c);
   switch(c) // action dépendant du caractère reçu 
    { 
     case '0':    //A1
#if defined(DEBUG)
Serial.print("O1-29G");
#endif
        action1();
        break; 
     case '1':    //B1
#if defined(DEBUG)
Serial.print("F1-29G");
#endif
        action2();
        break; 
     case '2':    //C1
#if defined(DEBUG)
Serial.print("F1-29D");
#endif
        action3();
        break; 
     case '3':    //D1
#if defined(DEBUG)
Serial.print("O1-29D");
#endif
        action4();
        break; 
     case '4':    //A2
#if defined(DEBUG)
Serial.print("O2-30G");
#endif
        action5();
        break; 
     case '5':    //B2
#if defined(DEBUG)
Serial.print("F2-30G");
#endif
        action6();
        break; 
     case '6':    //C2
#if defined(DEBUG)
Serial.print("F2-3031D");
#endif
        action7();
        break; 
     case '7':    //D2
#if defined(DEBUG)
Serial.print("O2-3031D");
#endif
        action8();
        break; 
     case '8':    //A3
#if defined(DEBUG)
Serial.print("O2-31G");
#endif
        action5();
        break; 
     case '9':    //B3
#if defined(DEBUG)
Serial.print("F2-31G");
#endif
        action6();
        break; 
    }            // end switch
 }
#endif    
}    // end LOOP

void action1()    // detecteur loin a gauche du PaN voie 1
{
/*- RA1 - O1-29G   -> si etat1=false, alors etat1=true, dir1=ADroite (entrée par la gauche de la voie 1, direction vers droite)
           -> si etat1=true, alors etat1=false,  (sortie par la gauche de la voie 1)
              et -> si etat2=false (voie 2 libre), alors etat_PAN=ouvert (ouverture PAN)
              */

  if (etat1 == 0)   // etat libre, entrée par la gauche voie 1
  {
    etat1++;     // voie 1 occupée
    dir1=true;  // vers la droite
  } else {        // sinon pas libre donc c'est une sortie vers la gauche
                  // ou une autre entree par la gauche
    if (!dir1)    // sortie
    {
      if (etat1 > 0) 
      {
        etat1--;  // voie 1 libre
      }
    } else {      // entree
      etat1++;
      dir1=true;
    }
    
    if ((etat1 == 0) && (etat2 == 0))  // si voies 1 et 2 libres
    {
      etat_PAN=true;  // ouverture si l'autre voie est libre
      FeuxON();
      OuvreBarriere();
    }
  }  
#if defined(DEBUG)
display();
#endif
}

void action2()    // detecteur pres à gauche du PaN voie 1
{
// RB1 - F1-29G   -> si etat1>0 et dir1=ADroite alors etat_PAN = ferme (fermeture PAN)

  if (etat1 && dir1 && etat_PAN) // vers la droite 2e détecteur à gauche du PaN
  {
    etat_PAN=false; // fermeture 
    FeuxON();
    FermeBarriere();
  }  
#if defined(DEBUG)
display();
#endif
}

void action3()    // detecteur près à droite du PaN voie 1
{
//- RC1 - F1-29D   -> si etat1>0 et dir1=AGauche alors etat_PAN = ferme (fermeture PAN)

  if (etat1 && !dir1 && etat_PAN)  // vers la gauche 2e détecteur à droite du PaN
  {
    etat_PAN=false; // fermeture
    FermeBarriere();
    FeuxON();
  }  
#if defined(DEBUG)
display();
#endif
}

void action4()    // detecteur loin a droite du PaN voie 1
{
/*- RD1 - O1-29D   -> si etat1=false, alors etat1=true, dir1=AGauche (entrée par la droite de la voie 1, direction vers gauche)
           -> si etat1=true,  alors etat1=false (sortie par la droite de la voie 1)
              et si etat2=false (voie 2 libre), alors etat_PAN=ouvert (ouverture PAN)
              */
              
  if (etat1 == 0)    // etat libre, entrée par la droite voie 1
  {
    etat1++;         // voie 1 occupee
    dir1=false;      // vers la gauche
  } else {           // sinon pas libre donc c'est une sortie vers la droite
                     // ou une nouvelle entree par le droite
    if (dir1)        // sortie
    {
      if (etat1 > 0)
      {
        etat1--;
      }
    } else {        // sinon  entree
      etat1++;
      dir1=false;
    }
    if ((etat1==0) && (etat2==0))
    {
      etat_PAN=true;
      FeuxON();
      OuvreBarriere();
    }
  }  
#if defined(DEBUG)
display();
#endif
}

void action5()
{
/*- RA2 ou RA3
- O2-30G   -> si etat2=false, alors etat2=true, dir2=ADroite (entrée par la gauche de la voie 2, direction vers droite)
- O2-31G   -> si etat2=true alors etat2=false (sortie par la gauche de la voie 2)
              et -> si etat1=false (voie 1 libre), alors etat_PAN=ouvert (ouverture PAN)
              */
              
  if (etat2 == 0)    // voie 2 libre, entree par la gauche
  {
    etat2++;
    dir2=true;
  } else {         // sinon pas libre donc c'est une sortie vers la gauche
                   // ou nouvelle entree par la gauche
    if (!dir2)      // sortie
    {
      if (etat2 > 0)
      {
        etat2--;
      }
    } else {      // sinon entree
      etat2++;
      dir2=true;
    }
    if ((etat1==0) && (etat2==0))
    {
      etat_PAN=true;
      OuvreBarriere();
    }
  }   
#if defined(DEBUG)
display();
#endif
}

void action6()
{
//- RB2 ou RB3 - F2-30G ou F2-31G   -> si etat2=true et dir2=ADroite alors etat_PAN = ferme (fermeture PAN)

  if (etat2 && dir2 && etat_PAN)
  {
    etat_PAN=false;
    FeuxON();
    FermeBarriere();
  }  
#if defined(DEBUG)
display();
#endif
}

void action7()
{
//- RC2 - F2-3031D -> si etat2=true et dir2=AGauche alors etat_PAN = ferme (fermeture PAN)

  if (etat2 && !dir2 && etat_PAN)
  {
    etat_PAN=false;
    FeuxON();
    FermeBarriere();
  }     
#if defined(DEBUG)
display();
#endif
}

void action8()
{
/*- RD2 
- O2-3031D -> si etat2=0, alors etat2=true, dir2=AGauche (entrée par la droite de la voie 2, direction vers gauche)
           -> si etat2=true  alors etat2=false (sortie par la droite de la voie 2 et )
              et -> si etat1=false (voie 1 libre), alors etat_PAN=ouvert (ouverture PAN)
              */
              
  if (etat2 == 0)    //entree par la droite, direction vers gauche
  {
    etat2++;
    dir2=false;
  } else {          // sinon sortie vers la droite
                    // ou nouvelle entree 
    if (dir2)
    {
      if (etat2 > 0)
      {
        etat2--;
      }
    } else {
      etat2++;
      dir2=false;
    }
    if ((etat1==0) && (etat2==0))
    {
      etat_PAN=true;
      FeuxON();
      OuvreBarriere();
    }
  }      
#if defined(DEBUG)
display();
#endif
}

void OuvreBarriere()
{
  digitalWrite(PAN1, LOW);  // commande relai sens ouvrir
  TimerPAN = millis();
  Fulgurex = true;
}

void FermeBarriere()
{
  digitalWrite(PAN2, LOW);  // commande relai sens ouvrir
  TimerPAN = millis();
  Fulgurex = true;
}


void FeuxON()
{
  switch (cycle)
  {
    case 0: //éteint >> démarrage croissance 
    cycle = 1;
    fading = 10;
    fadeValue = 10;
    break;
    
    case 1: //croissance
    analogWrite(Feux, fadeValue);
    fadeValue = fadeValue + 10;
    if (fadeValue > 255)
    {
      fading = 1000;
      cycle = 2;
    }
    break;
    
    case 2: //allumé >> démarrage décroissance
    cycle = 3;
    fading = 10;
    fadeValue = 255;
    break;
    
    case 3: // decroissance
    analogWrite(Feux, fadeValue);
    fadeValue = fadeValue - 10;
    if (fadeValue < 0)
    {
      fading = 500;
      cycle = 0;
    }
    break;
  }
}

void FeuxOFF()
{
  analogWrite(Feux, 0);    // éteint
}

/****************************************************************/
#if defined(DEBUG)
void display()        //debugging pour tester sur écran debug
{
  Serial.print(c);
  Serial.print(" occupation ");
  if (etat1)
  { 
    Serial.print("voie1 ");
    if (dir1) Serial.print("> "); else Serial.print("< ");
    Serial.print(etat1);
    Serial.print(" ");
  }
  if (etat2)
  { 
    Serial.print("voie2 ");
    if (dir2) Serial.print("> "); else Serial.print("< ");
    Serial.print(etat2);
    Serial.print(" ");
  }
  if ((etat1==0) && (etat2==0))
  {
    Serial.println("LIBRE");
  } else {
    Serial.println();
  }
}
#endif
